unit Fltexm1;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Fltcback, Grids, DBGrids, ExtCtrls, DBCtrls, DB, DBTables,
  StdCtrls;

{
  Remove this define if you want to test the filtering speed without using the
  filter record count function.  Most notably faster when filtering BLOB fields
}
{$define BDEFilterCount}

type
  TForm1 = class(TForm)
    Table1: TTable;
    DataSource1: TDataSource;
    DBNavigator1: TDBNavigator;
    DBGrid1: TDBGrid;
    FilterCallback1: TFilterCallback;
    Button1: TButton;
    RadioGroup1: TRadioGroup;
    Panel1: TPanel;
    Label1: TLabel;
    DBMemo1: TDBMemo;
    procedure Button1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure DBNavigator1Click(Sender: TObject; Button: TNavigateBtn);
  private
    { Private declarations }
    procedure NumRecords;

    procedure CommonName(FilterRec: TFilterObject; var Allow: Boolean);
    procedure SpeciesNo(FilterRec: TFilterObject; var Allow: Boolean);
    procedure LengthFlt(FilterRec: TFilterObject; var Allow: Boolean);
    procedure DescStartsWithV(FilterRec: TFilterObject; var Allow: Boolean);
    procedure CategoryCod(FilterRec: TFilterObject; var Allow: Boolean);
    procedure DateFilter(FilterRec: TFilterObject; var Allow: Boolean);
    procedure LengthRange(FilterRec: TFilterObject; var Allow: Boolean);
    procedure BlobFilter(FilterRec: TFilterObject; var Allow: Boolean);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.CommonName(FilterRec: TFilterObject;
  var Allow: Boolean);
begin
  Allow := False;
  with FilterRec do
  begin
    Allow := Pos('FISH', UpperCase(GetFieldData('Common_Name').AsString)) <> 0;
  end;
end;

procedure TForm1.SpeciesNo(FilterRec: TFilterObject;
  var Allow: Boolean);
begin
  Allow := False;
  with FilterRec do
  begin
    if IsValidField('Species No') then
      Allow := (GetFieldData('Species No').AsFloat > 90200.0);
  end;
end;

procedure TForm1.LengthFlt(FilterRec: TFilterObject;
  var Allow: Boolean);
begin
  Allow := False;
  with FilterRec do
  begin
    if IsValidField('Length (cm)') then
      Allow := (GetFieldData('Length (cm)').AsFloat < 50.0);
  end;
end;

procedure TForm1.DescStartsWithV(FilterRec: TFilterObject;
  var Allow: Boolean);
begin
  Allow := False;
  with FilterRec do
  begin
    if IsValidField('Species Name') then
      Allow := (Copy(GetFieldData('Species Name').AsString, 1, 1) = 'V');
  end;
end;

procedure TForm1.CategoryCod(FilterRec: TFilterObject;
  var Allow: Boolean);
begin
  Allow := False;
  with FilterRec do
  begin
    if IsValidField('Category') then
      Allow := (GetFieldData('Category').AsString = 'Wrasse');
  end;
end;

procedure TForm1.DateFilter(FilterRec: TFilterObject;
  var Allow: Boolean);
begin
  Allow := False;
  with FilterRec do
  begin
    if IsValidField('Date_Seen') then
      Allow := GetFieldData('Date_Seen').AsDate = EncodeDate(1997, 2, 10);
  end;
end;
procedure TForm1.LengthRange(FilterRec: TFilterObject;
  var Allow: Boolean);
var
  LN: Double;
begin
  Allow := False;
  with FilterRec do
  begin
    if IsValidField('Length (cm)') then
    begin
      LN  := GetFieldData('Length (cm)').AsFloat;
      Allow := (LN > 30.0) AND (LN < 100.0);
    end;
  end;
end;

(*
**  Filter routine for reading blob fields.  These are presented as streams.
**  When the GetBlobData is used, a temporary stream is created to the Fields
**  Blob data (an exception occurs if the field is not a blob field).  The stream
**  is released before the next record is passed in by the filter mechanism.  The
**  previous stream is also released if the GetBlobData call is issued again in
**  same filter routine.  This makes it possible to check multiple blob fields
**  data in the same routine.  However, only ONE may be open at any one time.
*)
procedure TForm1.BlobFilter(FilterRec: TFilterObject;
  var Allow: Boolean);
const
  BUFFSIZE = 1024;
var
  BuffChar: Pointer;
  Stream: TFilterBlobStream;
  NumBytes : Longint;
begin
  Allow := False;
  GetMem(BuffChar, BUFFSIZE);               { Get a buffer area for BLOB data }
  try
    FillChar(BuffChar^, BUFFSIZE, 0);       { Zero out buffer }
    with FilterRec do
    begin
      if IsValidBlobField('Notes') then     { Valid field? }
      begin
        Stream := GetBlobData('Notes');     { Associate streams }
        if Stream.Size >= BUFFSIZE then     { Entire blob is NOT searched! }
          ;                                 { Shows example of how to get blob size }
        NumBytes :=
          Stream.Read(BuffChar^, BUFFSIZE); { Read data }
        Allow :=
          StrPos(BuffChar, 'exposed') <> nil;{ 'exposed' in the data? }
      end
    end
  finally
    FreeMem(BuffChar, BUFFSIZE);            { Release buffer space }
  end;
end;

procedure TForm1.Button1Click(Sender: TObject);

  procedure SetOnFilterHandler(Index: byte);
  begin
    case Index of
      0:
        FilterCallback1.OnFilter := CommonName;
      1:
        FilterCallback1.OnFilter := SpeciesNo;
      2:
        FilterCallback1.OnFilter := LengthFlt;
      3:
        FilterCallback1.OnFilter := DescStartsWithV;
      4:
        FilterCallback1.OnFilter := CategoryCod;
      5:
        FilterCallback1.OnFilter := DateFilter;
      6:
        FilterCallback1.OnFilter := LengthRange;
      7:
        FilterCallback1.OnFilter := BlobFilter;
      else
        FilterCallback1.OnFilter := nil;
    end;
  end;

begin
  if FilterCallback1.Active then
  begin
    Button1.Caption := '&Apply';
    FilterCallBack1.EndFilter;
    RadioGroup1.Enabled := True;
    FilterCallback1.OnFilter := nil;
  end else
  begin
    Button1.Caption := '&Remove';
    SetOnFilterHandler(RadioGroup1.ItemIndex);  { Set BEFORE call to start }
    FilterCallBack1.StartFilter;
    RadioGroup1.Enabled := False;
  end;
  NumRecords;
  Table1.First;
end;

procedure TForm1.NumRecords;
begin
  {$ifdef BDEFilterCount}
  Label1.Caption := Format('Record Count: %d', [FilterCallback1.FilteredRecordCount]);
  {$else}
  Label1.Caption := '';
  {$endif}
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  with Table1 do
  begin
    Active := False;
    DatabaseName := ExtractFilePath(Application.ExeName);
    TableName := 'BIOLIFE2.DB';
    Active := True;
  end;
  NumRecords;
end;

procedure TForm1.DBNavigator1Click(Sender: TObject; Button: TNavigateBtn);
begin
  if Button = nbRefresh then
    NumRecords
end;

end.
